;;; DO NOT MODIFY THIS FILE
(if (featurep 'auctex-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "auctex/_pkg.el")

(package-provide 'auctex :version 1.42 :author-version "11.53" :type 'regular)

;;;***

;;;### (autoloads (turn-on-bib-cite bib-cite-minor-mode) "bib-cite" "auctex/bib-cite.el")

(autoload 'bib-cite-minor-mode "bib-cite" "\
Toggle bib-cite mode.
When bib-cite mode is enabled, citations, labels and refs are highlighted
when the mouse is over them.  Clicking on these highlights with [mouse-2]
runs bib-find, and [mouse-3] runs bib-display." t nil)

(autoload 'turn-on-bib-cite "bib-cite" "\
Unconditionally turn on Bib Cite mode." nil nil)

;;;***

;;;### (autoloads (context-en-mode) "context-en" "auctex/context-en.el")

(autoload 'context-en-mode "context-en" "\
Major mode for editing files for ConTeXt using its english interface.

Special commands:
\\{ConTeXt-mode-map}

Entering `context-mode' calls the value of `text-mode-hook',
then the value of TeX-mode-hook, and then the value
of context-mode-hook." t nil)

;;;***

;;;### (autoloads (context-nl-mode) "context-nl" "auctex/context-nl.el")

(autoload 'context-nl-mode "context-nl" "\
Major mode for editing files for ConTeXt using its dutch interface.

Special commands:
\\{ConTeXt-mode-map}

Entering `context-mode' calls the value of `text-mode-hook',
then the value of TeX-mode-hook, and then the value
of context-mode-hook." t nil)

;;;***

;;;### (autoloads (font-latex-setup) "font-latex" "auctex/font-latex.el")

(autoload 'font-latex-setup "font-latex" "\
Setup this buffer for LaTeX font-lock.  Usually called from a hook." nil nil)

;;;***

;;;### (autoloads (BibTeX-auto-store) "latex" "auctex/latex.el")

(autoload 'BibTeX-auto-store "latex" "\
This function should be called from `bibtex-mode-hook'.
It will setup BibTeX to store keys in an auto file." nil nil)

;;;***

;;;### (autoloads nil "tex-info" "auctex/tex-info.el")

;;;***

;;;### (autoloads nil "tex-jp" "auctex/tex-jp.el")
(autoload 'japanese-plain-tex-mode "tex-jp" nil t)
(autoload 'japanese-latex-mode "tex-jp" nil t)

;;;***

;;;### (autoloads (TeX-submit-bug-report TeX-insert-quote TeX-auto-generate-global TeX-auto-generate) "tex" "auctex/tex.el")

(autoload 'TeX-auto-generate "tex" "\
Generate style file for TEX and store it in AUTO.
If TEX is a directory, generate style files for all files in the directory." t nil)

(autoload 'TeX-auto-generate-global "tex" "\
Create global auto directory for global TeX macro definitions." t nil)
(autoload 'ams-tex-mode "tex" nil t)

(autoload 'TeX-insert-quote "tex" "\
Insert the appropriate quote marks for TeX.
Inserts the value of `TeX-open-quote' (normally ``) or `TeX-close-quote'
\(normally '') depending on the context.  If `TeX-quote-after-quote'
is non-nil, this insertion works only after \".
With prefix argument FORCE, always inserts \" characters." t nil)

(autoload 'TeX-submit-bug-report "tex" "\
Submit a bug report on AUCTeX via mail.

Don't hesitate to report any problems or inaccurate documentation.

If you don't have setup sending mail from (X)Emacs, please copy
the output buffer into your mail program, as is gives use
important information about your AUCTeX version and AUCTeX
configuration." t nil)

;;;***

;;;### (autoloads (texmathp) "texmathp" "auctex/texmathp.el")

(autoload 'texmathp "texmathp" "\
Determine if point is inside (La)TeX math mode.
Returns t or nil.  Additional info is placed into `texmathp-why'.
The functions assumes that you have (almost) syntactically correct (La)TeX in
the buffer.
See the variable `texmathp-tex-commands' about which commands are checked." t nil)

;;;***

(provide 'auctex-autoloads)
